
@file:Suppress("unused")

package ru.dbotthepony.kvector.api.concrete

import ru.dbotthepony.kvector.api.IFloatMatrix
import ru.dbotthepony.kvector.api.IMatrix
import ru.dbotthepony.kvector.api.ISquareMatrix
import ru.dbotthepony.kvector.api.ITransposable
import ru.dbotthepony.kvector.matrix.*
import ru.dbotthepony.kvector.vector.nfloat.*

/**
 * [Matrix2f] and [MutableMatrix2f] implement this
 */
interface IMatrix2f<T : IMatrix2f<T>> : IMatrix<T>, IFloatMatrix<T>, ITransposable<T> {
	val c00: Float
	val c10: Float
	val c01: Float
	val c11: Float
	val r00: Float
	val r01: Float
	val r10: Float
	val r11: Float
	val a11: Float
	val a21: Float
	val a12: Float
	val a22: Float
	val b11: Float
	val b21: Float
	val b12: Float
	val b22: Float
	val c0: Vector2f
	val c1: Vector2f
	val r0: Vector2f
	val r1: Vector2f
}

/**
 * [Matrix3f] and [MutableMatrix3f] implement this
 */
interface IMatrix3f<T : IMatrix3f<T>> : IMatrix<T>, IFloatMatrix<T>, ITransposable<T>, ISquareMatrix<T, Vector2f> {
	val c00: Float
	val c10: Float
	val c20: Float
	val c01: Float
	val c11: Float
	val c21: Float
	val c02: Float
	val c12: Float
	val c22: Float
	val r00: Float
	val r01: Float
	val r02: Float
	val r10: Float
	val r11: Float
	val r12: Float
	val r20: Float
	val r21: Float
	val r22: Float
	val a11: Float
	val a21: Float
	val a31: Float
	val a12: Float
	val a22: Float
	val a32: Float
	val a13: Float
	val a23: Float
	val a33: Float
	val b11: Float
	val b21: Float
	val b31: Float
	val b12: Float
	val b22: Float
	val b32: Float
	val b13: Float
	val b23: Float
	val b33: Float
	val c0: Vector3f
	val c1: Vector3f
	val c2: Vector3f
	val r0: Vector3f
	val r1: Vector3f
	val r2: Vector3f

	override val translation: Vector2f get() = Vector2f(r02, r12)
}

/**
 * [Matrix4f] and [MutableMatrix4f] implement this
 */
interface IMatrix4f<T : IMatrix4f<T>> : IMatrix<T>, IFloatMatrix<T>, ITransposable<T>, ISquareMatrix<T, Vector3f> {
	val c00: Float
	val c10: Float
	val c20: Float
	val c30: Float
	val c01: Float
	val c11: Float
	val c21: Float
	val c31: Float
	val c02: Float
	val c12: Float
	val c22: Float
	val c32: Float
	val c03: Float
	val c13: Float
	val c23: Float
	val c33: Float
	val r00: Float
	val r01: Float
	val r02: Float
	val r03: Float
	val r10: Float
	val r11: Float
	val r12: Float
	val r13: Float
	val r20: Float
	val r21: Float
	val r22: Float
	val r23: Float
	val r30: Float
	val r31: Float
	val r32: Float
	val r33: Float
	val a11: Float
	val a21: Float
	val a31: Float
	val a41: Float
	val a12: Float
	val a22: Float
	val a32: Float
	val a42: Float
	val a13: Float
	val a23: Float
	val a33: Float
	val a43: Float
	val a14: Float
	val a24: Float
	val a34: Float
	val a44: Float
	val b11: Float
	val b21: Float
	val b31: Float
	val b41: Float
	val b12: Float
	val b22: Float
	val b32: Float
	val b42: Float
	val b13: Float
	val b23: Float
	val b33: Float
	val b43: Float
	val b14: Float
	val b24: Float
	val b34: Float
	val b44: Float
	val c0: Vector4f
	val c1: Vector4f
	val c2: Vector4f
	val c3: Vector4f
	val r0: Vector4f
	val r1: Vector4f
	val r2: Vector4f
	val r3: Vector4f

	override val translation: Vector3f get() = Vector3f(r03, r13, r23)
}
